<?php
// $Id: oaiharvester.batch.php,v 1.1.2.15 2010/07/03 22:25:59 pkiraly Exp $

/**
 * @file
 * The functions need to run harvester.
 */

/**
 * Implementation of hook_cron().
 * run harvester periodically
 */
function oaiharvester_cron() {
  xc_log_info('harvester', 'cron started');
  variable_del('oaiharvester_statistics');
  $cron_last = variable_get('oaiharvester_cron_last', NULL);
  $never_run = FALSE;
  if (!is_numeric($cron_last)) {
    $never_run = TRUE;
    $cron_last = variable_get('install_time', 0);
  }
  //$cron_last = $cron_last - (60*60);
  $last = _parse_date($cron_last);
  $now  = _parse_date();

  $distance = $now->timestamp - $last->timestamp;
  xc_log_info('harvester', 'distance: ' . $distance);

  // TODO: handling minutes, only if the distance is less than an hour, else
  // we should run the hourly tasks anyway
  $hours = "";
  $minutes = "";
  if ($distance < OAIHARVESTER_MINUTE) {
    xc_log_info('harvester', 'distance is less than a minute');
    // do nothing
  }
  elseif ($distance < OAIHARVESTER_HOUR) {
    xc_log_info('harvester', 'distance is less than a hour');
    if ($last->hour != $now->hour) {
      $hours = sprintf("AND hour IN (%d, %d)", $last->hour, $now->hour);
      $minutes = sprintf("AND minute BETWEEN %d AND 59 OR minute BETWEEN 0 AND %d",
      $last->minute, $now->minute);
    }
    else {
      $hours = sprintf("AND hour = %d", $now->hour);
      $minutes = sprintf("AND minute BETWEEN %d AND %d", $last->minute, $now->minute);
    }
  }
  elseif ($distance < OAIHARVESTER_DAY) {
    xc_log_info('harvester', 'distance is less than a day');
    if ($last->hour > $now->hour) {
      $hours = sprintf("AND (hour BETWEEN %d AND 23 OR hour BETWEEN 0 AND %d)", $last->hour, $now->hour);
    }
    elseif ($last->hour < $now->hour) {
      $hours = sprintf("AND (hour BETWEEN %d AND %d)", $last->hour, $now->hour);
    }
  }
  
  // fix (2010-01-14): put $now >= start_date AND $now <= end_date 
  $sql = 'SELECT * FROM {oaiharvester_harvester_schedules}
    WHERE (CURDATE() >= DATE(start_date) AND CURDATE() <= DATE(end_date))
      AND ((recurrence = \'Weekly\' AND day_of_week IN (%d, %d) %s)
       OR (recurrence = \'Daily\' %s)
       OR (recurrence = \'Hourly\' %s))';
  xc_log_info('harvester', 'SQL: '. xc_util_get_sql($sql, $last->day_of_week, 
    $now->day_of_week, $hours, $hours, $minutes));
    
  $schedules = db_query(
    db_rewrite_sql($sql, '{oaiharvester_harvester_schedules}', 'harvest_schedule_id'),
    $last->day_of_week, 
    $now->day_of_week,
    $hours,
    $hours,
    $minutes
  );
  
  // put items to queue if this item is not already there
  while ($schedule = db_fetch_object($schedules)) {
    $schedule = _oaiharvester_schedule_get_full($schedule->harvest_schedule_id);
    
    $additional_batch_sets = module_invoke_all('oaiharvester_additional_harvest_steps', 
      $schedule->harvest_schedule_id);

    // put steps to queue
    $steps    = _oaiharvester_schedule_steps_get($schedule->harvest_schedule_id);
    $provider = oaiharvester_repository_load($schedule->provider_id);
    foreach ($steps as $step) {
      oaiharvester_add_item_to_queue(
        $schedule, 
        $step->name, // format name
        $step->set_spec, 
        _oaiharvester_format_date($step->last_ran, $provider->granularity)
      );
    }
    
    // TODO: set state active
  }
  
  $queue = _oaiharvester_queue_get();
  $operations = array();

  $saved_batch = new stdClass();
  drupal_write_record('oaiharvester_batch', $saved_batch);
  $operation_id = 0;
  
  while ($queue_item = db_fetch_object($queue)) {
    // run batch harvest
    xc_log_info('cron', 'launch ' . join(',', array($queue_item->provider_url, 
      $queue_item->metadata_prefix, $queue_item->set_name, 
      $queue_item->from_date)));

    $operations[] = oaiharvester_create_batch_harvester_operation(
      'cron',                       // launcher type, 
      $queue_item->harvest_id,      // launcher_id, 
      $queue_item->provider_url,    // url
      $queue_item->metadata_prefix, // metadataPrefix
      $queue_item->set_name,        // set
      $queue_item->from_date,       // from
      $queue_item->until_date,      // until
      $queue_item->parsing_mode,    // parsing_mode
      $saved_batch->id,             // saved batch id
      $operation_id++,              // operation count
      NULL                          // resumption token
    );
  }
  
  // call de-FRBR-ize index as the last task
  // if ($do_deferberize == TRUE) {
  //   $operations[] = array('xc_solr_defrbrzize_index', array('batch'));
  // }
  
  if (count($operations) > 0) {
    variable_set('oaiharvester_cron_last', time());
    oaiharvester_batch_set_add($operations);
    // remove duplicated values from the additional batch sets
    $additional_batch_sets = xc_util_array_unique($additional_batch_sets);
    foreach($additional_batch_sets as $batch_set) {
      for($i = 0; $i < count($batch_set['operations']); $i++) {
        array_push($batch_set['operations'][$i][1], $saved_batch->id, $operation_id++);
        array_push($operations, $batch_set['operations'][$i]);
      }
      batch_set($batch_set);
    }
    // save operations
    $saved_batch->sets = serialize($operations);
    drupal_write_record('oaiharvester_batch', $saved_batch, 'id');
    
    // Start!
    batch_process('harvester/schedule/list');
  }
} // oaiharvester_cron

/**
 * Manually launch the harvester.
 * @param $schedule (Object)
 *   The schedule object
 */
function oaiharvester_schedule_start($schedule) {
  xc_log_info('harvester', 'START manual harvesting');
  variable_del('oaiharvester_statistics');
  
  if (!isset($schedule->oai_provider_url)) {
    $schedule = _oaiharvester_schedule_get_full($schedule->harvest_schedule_id);
  }
  // TODO: set schedule state
  
  // do de-FRBR-ize index after harvesting?
  // $do_deferberize = ($schedule->do_defrbrize == 1) ? TRUE : FALSE;
  
  // get steps
  $steps = _oaiharvester_schedule_steps_get($schedule->harvest_schedule_id);
  $provider = oaiharvester_repository_load($schedule->provider_id);
  
  $saved_batch = new stdClass();
  drupal_write_record('oaiharvester_batch', $saved_batch);
  $operation_id = 0;

  $operations = array();
  foreach ($steps as $step) {
    $set_spec = empty($step->set_spec) ? NULL : $step->set_spec;
    $last_ran = _oaiharvester_format_date($step->last_ran, $provider->granularity);
    $operations[] = oaiharvester_create_batch_harvester_operation(
      'manual',                       // launcher_type
      $schedule->harvest_schedule_id, // launcher_id
      $schedule->oai_provider_url,    // url
      $step->name,                    // metadataPrefix
      $step->set_spec,                // set
      $last_ran,                      // from
      NULL,                           // until
      $schedule->parsing_mode,        // parsing_mode
      $saved_batch->id,               // saved batch id
      $operation_id++,                // operation count
      NULL                            // resumption token
    );
  }
  
  // add operations
  batch_set(oaiharvester_batch_set_create($operations));
  
  $additional_sets = module_invoke_all('oaiharvester_additional_harvest_steps', 
      $schedule->harvest_schedule_id);
  foreach($additional_sets as $batch_set) {
    // adding identifiers to operations, and adding additional operations to others 
    for($i = 0; $i < count($batch_set['operations']); $i++) {
      array_push($batch_set['operations'][$i][1], $saved_batch->id, $operation_id++);
      array_push($operations, $batch_set['operations'][$i]);
    }
    batch_set($batch_set);
  }
  // save operations
  $saved_batch->sets = serialize($operations);
  drupal_write_record('oaiharvester_batch', $saved_batch, 'id');
  
  $target_url = 'harvester/schedule/report/' . $saved_batch->id;

  // Start!
  batch_process($target_url);
} // oaiharvester_schedule_start

/**
 * Continue/restart harvesting after stoped.
 * @param $saved_batch (stdClass)
 *   The saved batch object, stored in DB
 * @param $operation (int)
 *   Which operation is the first, the program should restart/continue
 * @param $resumption_token (String)
 *   The last resumptionToken
 */
function oaiharvester_schedule_continue($saved_batch, $operation = 0, 
    $resumption_token = NULL) {
  xc_log_info('harvester', 'continue process');
  $sets = unserialize($saved_batch->sets);
  for ($i = $operation; $i < count($sets); $i++) {
    if ($i == $operation && $sets[$i][0] == 'oaiharvester_harvest_as_batch'
      && !is_null($resumption_token)) {
      // TODO: manipulate set: replace resumption token as the last parameter 
      $sets[$i][1][10] = base64_decode($resumption_token);
    }
    
    if ($sets[$i][0] == 'oaiharvester_harvest_as_batch') {
      $set = oaiharvester_batch_set_create(array(($sets[$i])));
    }
    elseif ($sets[$i][0] == 'xc_solr_defrbrzize_index') {
      $set = xc_oaiharvester_bridge_batch_set_create(array($sets[$i]));
    }
    else {
      drupal_set_message('problem', 'error');
    }
    batch_set($set);
  }
  $saved_batch->sets = serialize($sets);
  drupal_write_record('oaiharvester_batch', $saved_batch, 'id');
  
  $target_url = 'harvester/schedule/report/' . $saved_batch->id;
  batch_process($target_url);
}

function oaiharvester_schedule_restart($saved_batch, $operation) {
  echo 'Restart world!';
}

/**
 * Create an operation for batch harvesting 
 * @param $launcher_type The launcher's type ('manual' or 'cron')
 * @param $launcher_id The id of the launcher (schedule id or queue item id)
 * @param $url The URL of OAI data provider
 * @param $metadataPrefix The metadata prefix parameter for ListRecords call
 * @param $set The set specification
 * @param $from The from date parameter
 * @param $until The until date parameter
 * @return A simple array with key as the function 
 *   (oaiharvester_harvest_as_batch) and a value as the parameters of that 
 *   function
 */
function oaiharvester_create_batch_harvester_operation($launcher_type, 
    $launcher_id, $url, $metadataPrefix, $set = NULL, $from = NULL, $until = NULL, 
    $parsing_mode = 'dom', $saved_batch_id = NULL, $operation_id = NULL, 
    $resumption_token = NULL) {
  return array(
    'oaiharvester_harvest_as_batch',
    array(
      $launcher_type,
      $launcher_id,
      $url,
      $metadataPrefix,
      $set,
      $from,
      $until,
      $parsing_mode,
      $saved_batch_id,
      $operation_id,
      $resumption_token
    )
  );
}

/**
 * Launch a batch harvester, which use oaiharvester_harvest_as_batch funtion 
 * with different parameters to run multiple harvester schedules sequentially. 
 * When all finished, it calls the oaiharvester_harvest_finished functions, 
 * then redirects the browser to the list of schedules.
 * @param $operations (Array) 
 *   An array of operation. Each operation is a simple array, with a function 
 *   name, and the list of parameters. We use the oaiharvester_harvest_as_batch 
 *   funtion here.
 * @param $title (String) 
 *   The title of the progress page. Only safe strings should be passed.
 * @param $init_message (String) 
 *   The message displayed while the processing is initialized. Defaults to 
 *   t('Initializing.'). 
 * @param $progress_message (String) 
 *   Message displayed while processing the batch. Available placeholders are 
 *   @current, @remaining, @total, @percentage, @estimate and @elapsed. 
 *   Defaults to t('Completed @current of @total.'). 
 * @param $finishing_function (String)
 *   Name of a function to be executed after the batch has completed. This 
 *   should be used to perform any result massaging that may be needed, and 
 *   possibly save data in $_SESSION for display after final page redirection. 
 * @param $ending_url
 * @return unknown_type
 */
function _batch_harvester($operations, $title = 'Harvesting',
    $init_message = 'The harvesting process is beginning.', 
    $progress_message = 'Keep on running.', 
    $finishing_function = 'oaiharvester_harvest_finished',
    $ending_url = 'harvester/schedule/list') {

  variable_del('oaiharvester_statistics');
  oaiharvester_batch_set_add($operations, $title, $init_message, 
    $progress_message, $finishing_function);
  // only needed if not inside a form _submit handler :
  batch_process($ending_url);
}

/**
 * Add batch set to the batch process.
 * @param $operations (Array) An array of operation. Each operation is a simple 
 *   array, with a function name, and the list of parameters. We use the 
 *   oaiharvester_harvest_as_batch funtion here.
 * @param $title (String) The title of the progress page. Only safe strings 
     should be passed.
 * @param $init_message (String) The message displayed while the processing is 
     initialized. Defaults to t('Initializing.'). 
 * @param $progress_message (String) Message displayed while processing the 
     batch. Available placeholders are @current, @remaining, @total, @percentage, 
 *   @estimate and @elapsed. Defaults to t('Completed @current of @total.'). 
 * @param $finishing_function (String) Name of a function to be executed after 
 *   the batch has completed. This should be used to perform any result 
 *   massaging that may be needed, and possibly save data in $_SESSION for 
 *   display after final page redirection. 
 * @return unknown_type
 */
function oaiharvester_batch_set_add($operations, 
    $title = 'Harvesting and Processing',
    $init_message = 'The harvesting process is beginning.', 
    $progress_message = 'Keep on running.', 
    $finishing_function = 'oaiharvester_harvest_finished') {

  $batch = oaiharvester_batch_set_create($operations, $title, $init_message,
    $progress_message, $finishing_function);
  batch_set($batch);
}

/**
 * Add batch set to the batch process.
 * @param $operations (Array) An array of operation. Each operation is a simple 
 *   array, with a function name, and the list of parameters. We use the 
 *   oaiharvester_harvest_as_batch funtion here.
 * @param $title (String) The title of the progress page. Only safe strings 
 *   should be passed.
 * @param $init_message (String) The message displayed while the processing is 
 *   initialized. Defaults to t('Initializing.'). 
 * @param $progress_message (String) Message displayed while processing the 
 *   batch. Available placeholders are @current, @remaining, @total, @percentage, 
 *   @estimate and @elapsed. Defaults to t('Completed @current of @total.'). 
 * @param $finishing_function (String) Name of a function to be executed after 
 *   the batch has completed. This should be used to perform any result 
 *   massaging that may be needed, and possibly save data in $_SESSION for 
 *   display after final page redirection. 
 * @return unknown_type
 */
function oaiharvester_batch_set_create($operations, 
    $title = 'Harvesting',
    $init_message = 'The harvesting process is beginning.', 
    $progress_message = '', 
    $finishing_function = 'oaiharvester_harvest_finished') {

  $batch = array(
    'operations'       => $operations,
    'title'            => t($title),
    'init_message'     => t($init_message),
    'progress_message' => t($progress_message),
    'finished'         => $finishing_function,
  );
  return $batch;
}


/**
 * Harvest the records in batch mode
 * @param $launch_type (String) 
 *   The type of lauch. ('manual' or 'cron')
 * @param $launcher_id (int) 
 *   The id of the launcher (schedule id or queue id)
 * @param $url (String) 
 *   The base URL to harvest
 * @param $metadataPrefix (String) 
 *   The OAI metadataPrefix parameter
 * @param $set (String)
 *   The OAI set parameter
 * @param $from (Date String)
 *   The OAI from parameter
 * @param $until (Date String)
 *   The OAI until parameter
 * @param $parsing_mode (String) 
 *   The mode of parsing of XML responses come from OAI server (possible values:
 *   'dom' or 'regex')
 * @param $context (Array)
 *   A context array to store different information. The $context['results'] is
 *   the same as the finished function's $result parameter.
 */
function oaiharvester_harvest_as_batch($launch_type = 'manual', 
    $launcher_id = NULL, $url = NULL, $metadataPrefix = NULL, $set = NULL, 
    $from = NULL, $until = NULL, $parsing_mode = 'dom', $saved_batch_id, 
    $operation_id, $resumptionToken = NULL, &$context = array()) {
  global $_oaiharvester_statistics;
  
  set_time_limit(0);
  $current_batch = batch_get();
  $_oaiharvester_statistics = variable_get('oaiharvester_statistics', array('start' => microtime(TRUE)));

  $schedule_id = 0;
  if ($launch_type == 'manual') {
    $schedule_id = $launcher_id;
  }
  elseif ($launch_type == 'cron') {
    $queue_item = _oaiharvester_queue_get_item_by_id($launcher_id);
    $schedule_id = $queue_item->harvest_schedule_id;
  }
  $schedule = oaiharvester_schedule_load($schedule_id);
  
  $saved_batch = oaiharvester_saved_batch_load($saved_batch_id, 'sets');
  $is_first = FALSE;
  if (empty($context['sandbox'])) {
    $is_first = TRUE;
    $context['results'] = array(
      // storing parameters
      'parameters' => array(
        'launch_type'   => $launch_type,
        'launcher_id'   => $launcher_id,
        'url'           => $url,
        'metadataPrefix'=> $metadataPrefix,
        'set'           => $set,
        'from'          => $from,
        'until'         => $until,
        'parsing_mode'  => $parsing_mode,
        'saved_batch_id'=> $saved_batch_id,
        'operation_id'  => $operation_id,
        'resumptionToken' => $resumptionToken,
        'max_request'   => 1,
      ),

      // storing parameters
      'count'              => 0,
      'request_count'      => 0,
      'start_time'         => microtime(TRUE),
      'total_time'         => 0.0,
      'foreach_time'       => 0.0,
      'request_time'       => 0.0,
      'oai_build_args'     => 0.0,
      'oai_response'       => 0.0,
      'oai_dom'            => 0.0,
      'resp_validate_verb' => 0.0,
      'resp_validate_url'  => 0.0,
      'resp_http_response' => 0.0,
      'resp_curl_init'     => 0.0,
      
      // computed values
      'saved_batch'        => $saved_batch,
      'schedule_id'        => $schedule_id,
    ); 
    
    $context['sandbox']['progress']       = 0;
    $context['sandbox']['start_time']     = time();
    $context['sandbox']['metadataPrefix'] = $metadataPrefix;
  }
  if ($is_first == TRUE && is_null($resumptionToken)) {
    $params = array(
      'metadataPrefix' => $metadataPrefix
    );
    if (!empty($set)) {
      $params['set'] = $set;
    }
    if (!empty($from)) {
      $params['from'] = $from;
    }
    if (!empty($until)) {
      $params['until'] = $until;
    }
  }
  else {
    $params = array(
      'resumptionToken' => (is_null($resumptionToken) 
        ? $context['results']['parameters']['resumptionToken']
        : $resumptionToken),
    );
  }
  $context['results']['request_count']++;
  $request = $url . '?' . http_build_query(array_merge(array('verb' => 'ListRecords'), $params));
  xc_log_info('harvester', 'Start downloading %url', array('%url' => $request));

  $context['message'] = ''; //$request;
  $t0 = microtime(TRUE);
  $download_phase_successfull = FALSE;
  try {
    //xc_log_info('harvester', 'Getting %url.', array('%url' => $request));
    // THE CACHING IS ONLY FOR TESTING PURPOSES. 
    // If you want to use, insert next line at the end of sites/default/settings.php:
    // $conf['oaiharvester_do_cache'] = TRUE;
    $http_cache = FALSE;
    if ($schedule->is_cacheable == 1) {
      // TODO: first check whether oaiharvester_http_cache is exists
      // then create, set permissions etc.
      $file_dir  = file_directory_path();
      $cache_dir = $file_dir . '/oaiharvester_http_cache/';
      $http_cache = $cache_dir . $schedule->harvest_schedule_id;
      if($is_first == TRUE) {
        if(!file_check_directory($cache_dir, FILE_CREATE_DIRECTORY)) {
          xc_log_error('harvester', t('The Drupal default \'file\' directory is
            not writable. Check out your Drupal settings, or turn off the 
            caching of harvested responses. Your current file directory is 
            %file.', array('%file' => $file_dir)));
        }
        else {
          $file_check_success = file_check_directory($http_cache, FILE_CREATE_DIRECTORY);
          if(!$file_check_success) {
            xc_log_error('harvester', t('The OAI harvester\'s cache directory
              (%file) is not writable. Check out your Drupal settings, or turn 
              off the caching of harvested responses.', 
              array('%file' => $cache_dir)));
          }
        }
      }
      else {
        $file_check_success = file_check_directory($http_cache);
      }
      if(!$file_check_success) {
        $http_cache = FALSE;
      }
    }
    if ($parsing_mode == 'regex') {
      $oaiPmh = new OaiPmh('ListRecords', $url, $params, $http_cache);
      $responseObject = $oaiPmh->getResponseObject();
      $t01 = microtime(TRUE);
      $parser = new OAIRegexParser();
      //xc_log_info('harvester', 'start process');
      $parser->process(
        $responseObject->getResponse(), 
        $responseObject->getHttpCode()
      );
      if ($parser->has_errors()) {
        foreach ($parser->get_internal_errors() as $error) {
          xc_log_error('harvester', 'regex parser error: ' . $error);
        }
        throw new Exception(join(', ', $parser->get_internal_errors()));
      }
      $statistics = $oaiPmh->getStatistics();
      $statistics['oai_dom'] = microtime(TRUE) - $t01;
    }
    else {
      $parser = new OAIDomParser(OaiPmh::factory('ListRecords', $url, $params, 
        $http_cache));
      $parser->set_http_code(OaiPmh::getStaticHttpCode());
      $statistics = OaiPmh::getStaticStatistics();
    }
    $t1 = microtime(TRUE);
    $context['results']['request_time']  += $t1 - $t0;
    
    if ((int)$parser->get_http_code() != 200) {
      $msg = t('The data provider had some trouble (HTTP %code) with responding'
        . 'the request !r', array('%code' => $parser->get_http_code(), 
        '!r' => l($request, str_replace('&amp;', '&', $request))));
      drupal_set_message($msg, 'error');
      xc_log_error('harvester', $msg);
      $context['results'] = $msg;
      $context['finished'] = 1;
    }
    else { // HTTP 200
      $context['results'] = array_merge($context['results'], $statistics);
      $download_phase_successfull = TRUE;
    }
  }
  catch (Exception $e) {
    $t1 = microtime(TRUE);
    $context['results']['request_time'] += $t1 - $t0;
    $msg = t('Communication error during harvesting: %m (%r).', 
             array('%m' => $e->getMessage(), '%r' => $request));
    $context['results']['error'] = $msg;
    drupal_set_message($msg, 'error');
    xc_log_error('harvester', $msg);
    $context['finished'] = 1;
  }

  if ($download_phase_successfull) {
    //xc_log_info('harvester', 'download_phase_successfull');
    
    if ($parser->has_errors()) {
      //xc_log_info('harvester', 'has errors');
      foreach ($parser->list_errors() as $error) {
        if ($error['attributes']['code'] == 'noRecordsMatch') {
          $msg = t('WARNING! No records match the requested criteria. '
            . 'Repository\'s message: %t. URL: %u', 
            array(
              '%t' => $error['text'],
              '%u' => $request
          ));
          $context['results']['error'] = $msg;
          drupal_set_message($msg, 'warning');
          xc_log_warning('harvester', $msg);
        }
        else {
          $msg = t('Error while harvesting. %c: %t (%u)', 
            array(
              '%c' => $error['attributes']['code'], 
              '%t' => $error['text'],
              '%u' => $request
          ));
          $context['results']['error'] = $msg;
          drupal_set_message($msg, 'error');
          xc_log_error('harvester', $msg);
        }
      }
      $context['finished'] = 1;
    }
    // has no errors
    else {
      if ($parser->has_records()) {
        $context['results']['count'] += $parser->get_record_count();
        $tf0 = microtime(TRUE);
        while ($record = $parser->get_next_record()) {
          module_invoke_all('oaiharvester_process_record', $record, $schedule_id);
        }
        if ($parsing_mode == 'regex') {
          $statistics['regex'] = $parser->getStatistics();
        }
        $context['results']['current_foreach_time'] = microtime(TRUE) - $tf0;
        $context['results']['foreach_time'] += $context['results']['current_foreach_time'];
      }
      //xc_log_info('harvester', '->oaiharvester_request_processed');
      module_invoke_all('oaiharvester_request_processed', 
        $context['results']['parameters'], $context['sandbox']);

      // preparing for the next request
      $context['finished'] = 0.5;
      $context['message'] .= t('Step %current of %total: Harvesting Metadata<br/>', 
        array(
          '%current' => $operation_id + 1, // because operation_id starts from 0
          '%total' => count($saved_batch),
      ));
      if ($parser->has_resumptionToken()) {
        $resumptionToken = $parser->get_resumptionToken();
        $time_elapsed      = (int)time() - (int)$context['sandbox']['start_time'];
        $current_process   = (microtime(TRUE) - $t0);
        $time_remained = -1;
        $complete_list_size = t('unkown number of');
        $number_of_records = $context['results']['count'];
        if (isset($resumptionToken['attributes'])) {
          $att = $resumptionToken['attributes'];
          // handling cursor position if exists
          if (isset($att['completeListSize']) && $att['completeListSize'] != 0) {
            $complete_list_size = $att['completeListSize'];
            if (!empty($att['cursor'])) {
              $number_of_records = $att['cursor'] + $parser->get_record_count();
            }
            $percent           = $number_of_records / $complete_list_size;
            $time_remained     = (int)($time_elapsed / $percent) - $time_elapsed;
            $context['finished'] = $percent;
          }
        }
        $context['message'] .= t('Harvested %num of %tot records.<br/>' .
          'Elapsed time: %el, remained approx. %re, total approx.: %ato.<br/>',
          array(
            '%num' => $number_of_records,
            '%tot' => $complete_list_size,
            '%el'  => oaiharvester_sec_to_time($time_elapsed),
            '%re'  => ($time_remained != -1) 
              ? oaiharvester_sec_to_time($time_remained) : t('unknown'),
            '%ato' => ($time_remained != -1)
              ? oaiharvester_sec_to_time($time_elapsed + $time_remained)
              : t('unknown'),
        ));
        xc_log_info('oaiharvester', $context['message']);
        $_oaiharvester_statistics['oai_response'] += $statistics['oai_response'];
        $_oaiharvester_statistics['http_response'] += $statistics['resp_http_response'];
        $_oaiharvester_statistics['oai_dom'] += $statistics['oai_dom'];
        $message = t('Details of current request (in sec):
          whole process: %whl, response: %or (%per-or%) (http: %http), 
          parse whole: %od (%per-od%), and individual records: %cft (%per-cft%)',
          array(
            '%whl'    => sprintf('%.3f', $current_process),
            '%or'     => sprintf('%.3f', $statistics['oai_response']),
            '%per-or' => sprintf('%.1f', ($statistics['oai_response'] * 100 
              / $current_process)),
            '%http'   => sprintf('%.3f', $statistics['resp_http_response']),
            '%od'     => sprintf('%.3f', $statistics['oai_dom']),
            '%per-od' => sprintf('%.1f', ($statistics['oai_dom'] * 100 
              / $current_process)),
            '%cft'    => sprintf('%.3f', $context['results']['current_foreach_time']),
            '%per-cft' => sprintf('%.1f', ($context['results']['current_foreach_time'] 
              * 100 / $current_process)),
        ));
        if ($parsing_mode == 'regex') {
          $message .= ' ' . t('new: %new (%sch, %dom), parse: %prs.)',
            array(
              '%new'  => sprintf('%.3f', $statistics['regex']['foreach']['new']),
              '%prs'  => sprintf('%.3f', $statistics['regex']['foreach']['prs']),
              '%sch'  => sprintf('%.3f', $statistics['regex']['foreach']['dom-set-schemas']),
              '%dom'  => sprintf('%.3f', $statistics['regex']['foreach']['dom-set-dom']),
          ));
        }
        xc_log_info('oaiharvester', $message);
        $context['message'] .= t('Warning! Do not close this window untill '
          . 'harvesting is complete!<br/>');
        $link = l(t('Stop harvesting'), //'harvester/schedule/' . $schedule_id);
          'batch', //harvester/schedule/report/' . $saved_batch_id, 
           array('query' => array('op' => 'finished', 
          'id' => $current_batch['id'])));
        $context['message'] .= ' ' . $link . ' '
          . t('(You will be able to restart or start over.)');
        xc_log_info('harvester', 'Next resumption token is: ' . $resumptionToken['text']);
        $context['results']['parameters']['resumptionToken'] = $resumptionToken['text'];
      }
      // no more resumptionToken
      else {
        xc_log_info('harvester', 'No more records to harvest');
        $context['finished'] = 1;
        // TODO: put an 'end of step' function to set the end date
      } // no more resumptionToken
    } // no errors
  } // download phase was successfull
  
  // TODO: use the following code to harvest only the first n lines.
  /*
  xc_log_info('harvester', 'request_count: ' . $context['results']['request_count'] . ', max_request: ' . $context['results']['parameters']['max_request']);
  if($context['results']['request_count'] >= $context['results']['parameters']['max_request']) {
    $context['finished'] = 1;
  }
  */
  
  if ($context['finished'] == 1) {
    $context['results']['end_time']   = microtime(TRUE);
    $context['results']['total_time'] = $context['results']['end_time'] 
      - $context['results']['start_time'];
    xc_log_info('harvester', 'download_phase_successfull: ' 
      . (int)$download_phase_successfull);
    _oaiharvester_step_processed(
      (!$download_phase_successfull || $parser->has_errors()),
      $context['results']['parameters'],
      $context['sandbox']['start_time']
    );
  }
  variable_set('oaiharvester_statistics', $_oaiharvester_statistics);
  xc_log_info('harvester', 'Finished processing request %m. Processing rate: %r', 
    array('%r' => $context['finished'], '%m' => $context['message']));
}

/**
 * The function run after the main harvester batch operations.
 * 
 * It invokes the hook_oaiharvester_harvest_finished() hook.
 * 
 * @param $success (Boolean)
 *   TRUE if the batch operations were run correctly, FALSE, is it ran wrongly
 *   or interrupted
 * @param $results (Array)
 *   The content of the batch's $context['results'] array
 * @param $operations (Array)
 *   The remaining operations
 */
function oaiharvester_harvest_finished($success, $results, $operations) {
  // TODO: update $repository->last_harvest_end_time
  if(!isset($results['end_time'])) {
    $results['end_time'] = microtime(TRUE);
  }
  $saved_batch_id = $results['parameters']['saved_batch_id'];
  $operation_id   = $results['parameters']['operation_id'];
  $saved_batch = oaiharvester_saved_batch_load($saved_batch_id);
  $saved_report = empty($saved_batch->reports) 
    ? array() 
    : unserialize($saved_batch->reports);
  
  if($success) {
    if(!empty($results['error'])) {
      drupal_set_message('ERROR: ' . $results['error'], 'error');
    }
    $saved_report['results'][] =  t('%count records were harvested in %total_time',
      array('%count' => $results['count'], 
        '%total_time' => oaiharvester_sec_to_time($results['total_time'])));
    $saved_report['tech_details'][] = t('Details of harvesting: ...');
  }
  else {
    if(!empty($results['error'])) {
      $saved_report['errors'] = $results['error'];
    }
    $saved_operations = unserialize($saved_batch->sets);
    // TODO: how to continue harvesting
    $continuable = FALSE;
    //if ($continuable) {
      $saved_report['actions'][] = l('Continue harvesting', 
        'harvester/schedule/continue/' . $saved_batch_id . '/' . $operation_id
        . '/' . base64_encode($results['parameters']['resumptionToken'])
      );
    //}
    //else {
      $saved_report['actions'][] = l('Restart harvesting', 
        'harvester/schedule/continue/' . $saved_batch_id . '/' . $operation_id);
    //}
    for ($i = $operation_id; $i < count($saved_operations); $i++) {
      $saved_report['pending_operations'][] = $saved_operations[$i][0]; // . '(' . join(', ', $saved_operations[$i][1]) . ')';
      if ($saved_operations[$i][0] == 'xc_solr_defrbrzize_index') {
        $location = $saved_operations[$i][1][0];
        $saved_report['actions'][] = l(t('run Solr normalization'), 
          'xc/solr/defrbrize_index/' . 
          (!is_null($location) ? $location->location_id : ''));
      }
    }
  }
  // TODO: find out how to delete by URL
  if(function_exists('_schedule_id_to_source_id')) {
    $source_id = _schedule_id_to_source_id($results['schedule_id']);
    $saved_report['actions'][] = l('Delete harvested records', 
    'xc/metadata/delete_source/' . $source_id 
    . '/' . (int)$results['start_time'] . '/' . (int)$results['end_time']);
  }
  $saved_report['actions'][] = t('Go to the !url', 
    array('!url' => l(t('schedule list'), 'harvester/schedule/list')));
  $saved_batch->reports = serialize($saved_report);
  $ret = drupal_write_record('oaiharvester_batch', $saved_batch, 'id');
  
  $statistics = t('results: %count  records were harvested' 
    . ', the type of lauch was %launch_type'
    . ', id of laucher was %launcher_id'
    . ', total time: %total_time'
    . ', validate verb: %resp_validate_verb'
    . ', validate url: %resp_validate_url'
    . ', args time: %oai_build_args' 
    . ', curl init: %resp_curl_init'
    . ', request time: %request_time' 
    . ', response time: %oai_response'
    . ', dom time: %oai_dom'
    . ', foreach time: %foreach_time' 
    . ', response: %resp_http_response',
    array(
      '%count'              => $results['count'],
      '%launch_type'        => $results['parameters']['launch_type'],
      '%launcher_id'        => $results['parameters']['launcher_id'],
      '%total_time'         => sprintf("%01.2f", $results['total_time']),
      '%request_time'       => sprintf("%01.2f", $results['request_time']), 
      '%oai_build_args'     => sprintf("%01.2f", $results['oai_build_args']), 
      '%oai_response'       => sprintf("%01.2f", $results['oai_response']), 
      '%oai_dom'            => sprintf("%01.2f", $results['oai_dom']),
      '%resp_validate_verb' => sprintf("%01.2f", $results['resp_validate_verb']), 
      '%resp_validate_url'  => sprintf("%01.2f", $results['resp_validate_url']), 
      '%foreach_time'       => sprintf("%01.2f", $results['foreach_time']),
      '%resp_http_response' => sprintf("%01.2f", $results['resp_http_response']),
      '%resp_curl_init'     => sprintf("%01.2f", $results['resp_curl_init'])
    )
  );
  xc_log_info('oaiharvester', $statistics);
  $process_stat = variable_get('oaiharvester_statistics', array());
  $process_stat['total'] = microtime(TRUE) - $process_stat['start'];
  unset($process_stat['start']);
  if(!empty($process_stat)) {
    arsort($process_stat, SORT_NUMERIC);
    $statistics = array();
    foreach ($process_stat as $key => $value) {
      //if ($key == 'store') {
      //  $key .= ' (total)';
      //}
      $statistics[] = t('%key: %value (%per%)', array(
        '%key' => $key,
        '%value' => sprintf("%01.3f", $value),
        '%per' => ($process_stat['total'] == 0) ? 0 
          : sprintf("%01.3f", ($value / $process_stat['total']) * 100)));
    }
    xc_log_info('oaiharvester', 'statistics: ' . theme('item_list', $statistics));
  }
  // drupal_set_message(t('operations: %op', array('%op' => var_export($operations, TRUE))));
  xc_log_info('harvester', 'Harvesting has finished');
  
  module_invoke_all('oaiharvester_harvest_finished', $success, $results, 
    $operations);
} // oaiharvester_harvest_finished

/**
 * Load the saved batch from DB
 * @param $id (int) 
 *   The ID of the record
 * @param $field (string)
 *   Which field's content should be return? If NULL (default), the whole object
 *   will be returned
 * @return unknown_type
 *   If specify a field, the field's content will be returned, else (default), 
 *   the whole object
 */
function oaiharvester_saved_batch_load($id, $field = NULL) {
  $sql = "SELECT * FROM {oaiharvester_batch} WHERE id = %d";
  $result = db_query(db_rewrite_sql($sql, '{oaiharvester_batch}', 'id'), $id);
  $data = db_fetch_object($result);
  if (!empty($field) && in_array($field, array('sets', 'reports'))) {
    return unserialize($data->$field);
  }
  else {
    return $data;
  }
} // oaiharvester_get_saved_batch

/**
 * report, actions, tech_details
 * @param $saved_batch
 * @return unknown_type
 */
function oaiharvester_schedule_report($oaiharvester_saved_batch) {
  if(empty($oaiharvester_saved_batch)) {
    return '';
  }
  $report = unserialize($oaiharvester_saved_batch->reports);
  $categories = array(
    'errors' => t('Errors'), 
    'pending_operations' => t('Pending operations'), 
    'results' => t('Results'), 
    'actions' => t('What do you want to do?'), 
    'tech_details' => t('Technological details'),
  );
  $output = '';
  foreach ($categories as $category => $label) {
    if (!empty($report[$category])) {
      $output .= theme('item_list', array_unique($report[$category]), $label);
    }
  }
  return $output;
}